@@ -15,8 +15,14 @@ class BraceExpand(unittest.TestCase):
             ('{10..7}', ['10', '9', '8', '7']),
             ('{1..5..2}', ['1', '3', '5']),
             ('{5..1..2}', ['5', '3', '1']),
+            ('{1..3..0}', ['1', '2', '3']),
+            ('{1..3..-0}', ['1', '2', '3']),
             ('{07..10}', ['07', '08', '09', '10']),
             ('{7..010}', ['007', '008', '009', '010']),
+            ('{1..-2}', ['1', '0', '-1', '-2']),
+            ('{01..-2}', ['01', '00', '-1', '-2']),
+            ('{1..-02}', ['001', '000', '-01', '-02']),
+            ('{-01..3..2}', ['-01', '001', '003']),
             ('{a..e}', ['a', 'b', 'c', 'd', 'e']),
             ('{a..e..2}', ['a', 'c', 'e']),
             ('{e..a}', ['e', 'd', 'c', 'b', 'a']),
@@ -28,6 +34,9 @@ class BraceExpand(unittest.TestCase):
             ('{,}', ['', '']),
             ('{Z..a}', ['Z', 'a']),
             ('{a..Z}', ['a', 'Z']),
+            ('{a.{b,c}}', ['{a.b}', '{a.c}']),
+            ('{a.{1..2}}', ['{a.1}', '{a.2}']),
+            ('{{{,}}}', ['{{}}', '{{}}']),
     ]
 
     unbalanced_tests = [
@@ -113,6 +122,15 @@ class BraceExpand(unittest.TestCase):
         result = list(braceexpand('{10..0}'))
         self.assertEqual(result[-2:], ['1', '0'])
 
+    def test_negative_zero(self):
+        result = list(braceexpand('{-0..1}'))
+        self.assertEqual(result, ['0', '1'])
+
+        result = list(braceexpand('{1..-0}'))
+        self.assertEqual(result, ['1', '0'])
+
+        result = list(braceexpand('{0..-0}'))
+        self.assertEqual(result, ['0'])
 
 if __name__ == '__main__':
     unittest.main()
